import requests
import os
import time

def get_with_retry(url, headers, max_retries=3):
    retries = 0
    while retries < max_retries:
        response = requests.get(url, headers=headers)
        if response.status_code != 429:
            return response
        wait_time = (2 ** retries) + 2  # Exponential backoff with minimum 2 seconds
        print(f"Rate limited. Retrying in {wait_time} seconds...")
        time.sleep(wait_time)
        retries += 1
    return response

def get_titled_players(headers):
    titles = ['GM', 'WGM', 'IM', 'WIM', 'FM', 'WFM', 'NM', 'WNM', 'CM', 'WCM']
    players = set()
    
    for title in titles:
        url = f"https://api.chess.com/pub/titled/{title}"
        response = get_with_retry(url, headers)
        
        if response.status_code == 200:
            data = response.json()
            players.update(data.get('players', []))
            print(f"Found {len(data.get('players', []))} {title} players")
        else:
            print(f"Error {response.status_code} fetching {title}")
        
        time.sleep(1)  # Respectful delay between title requests
    
    return players

def download_player_games(username, headers, output_dir, target_year, target_month):
    archives_url = f"https://api.chess.com/pub/player/{username}/games/archives"
    response = get_with_retry(archives_url, headers)
    
    if response.status_code != 200:
        print(f"  Skipping {username}: No archives found ({response.status_code})")
        return
    
    try:
        archives = response.json().get('archives', [])
    except ValueError:
        print(f"  Invalid response for {username}'s archives")
        return

    for archive_url in archives:
        # Extract year and month from archive URL
        parts = archive_url.rstrip('/').split('/')
        if len(parts) < 2:
            print(f"  Invalid archive URL format: {archive_url}")
            continue
        url_year = parts[-2]
        url_month = parts[-1]
        
        if url_year != target_year or url_month != target_month:
            continue  # Skip non-target month
        
        pgn_url = f"{archive_url}/pgn"
        response_pgn = get_with_retry(pgn_url, headers)
        
        if response_pgn.status_code == 200:
            content_disp = response_pgn.headers.get('Content-Disposition', '')
            filename = (
                content_disp.split('filename=')[1].strip('"') 
                if 'filename=' in content_disp 
                else f"{username}_{url_year}{url_month}.pgn"
            )
            filepath = os.path.join(output_dir, filename)
            
            if not os.path.exists(filepath):
                with open(filepath, 'w', encoding='utf-8') as f:
                    f.write(response_pgn.text)
                print(f"  Saved {filename}")
            else:
                print(f"  Exists {filename}")
        else:
            print(f"  Failed to download {pgn_url} ({response_pgn.status_code})")
        
        time.sleep(1)  # Delay between PGN downloads

def main():
    headers = {
        'User-Agent': 'Chess.com Titled Games Collector (contact@example.com)'
    }
    output_base_dir = "chesscom-titled"
    
    # Get target year and month from user
    year = input("Enter the year (YYYY): ").strip()
    while len(year) != 4 or not year.isdigit():
        print("Invalid year. Please enter a 4-digit year.")
        year = input("Enter the year (YYYY): ").strip()
    
    while True:
        month_input = input("Enter the month (1-12): ").strip()
        if not month_input.isdigit() or not (1 <= int(month_input) <= 12):
            print("Invalid month. Please enter a number between 1 and 12.")
            continue
        month = f"{int(month_input):02d}"
        break
    
    output_dir = os.path.join(output_base_dir, year, month)
    os.makedirs(output_dir, exist_ok=True)
    
    # Get all titled players
    print("Fetching titled players...")
    titled_players = get_titled_players(headers)
    print(f"\nTotal unique titled players: {len(titled_players)}\n")
    
    # Download games for each player
    for idx, username in enumerate(titled_players, 1):
        print(f"{idx}/{len(titled_players)}: Processing {username}")
        download_player_games(username, headers, output_dir, year, month)
        time.sleep(1)  # Delay between player processing

if __name__ == "__main__":
    main()